# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
from django.core.management.base import BaseCommand
from django.core.urlresolvers import reverse
from ...models import Token
from updater.conf import settings
from django.contrib.sites.shortcuts import get_current_site
import requests
from requests.exceptions import RequestException
import sys


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument("--token", default=False, type=str)

    def handle(self, *args, **options):

        if not settings.UPDATER_TOKEN and not options["token"]:
            self.stdout.write("whoop, need token")
            return

        # create the view token
        if not Token.objects.all().exists():
            Token.objects.create()

        token = Token.objects.all().first().token

        updater_token = options["token"] or settings.UPDATER_TOKEN

        # building urls
        domain = get_current_site(None)

        url = hit_updater_view(domain, token=token)

        if not url:
            self.stdout.write("Unable to find the correct domain name for this installation, tried {0}".format(domain))
            self.stdout.write("Please note: The Django Updater service won't work on a dev environment.")

        while not url:
            try:
                domain = get_input("Domain: ")
            except KeyboardInterrupt:
                return
            url = hit_updater_view(domain=domain, token=token)

        base_url = url.split("/updater/")[0]
        self.stdout.write("This site is reachable at {base}".format(base=base_url))

        self.stdout.write("Contacting online service to register this site.")

        data = {"name": domain, "url": url}
        headers = {"Authorization": "Token " + updater_token}
        r = requests.post(settings.UPDATER_BASE_URL + "/api/v1/sites/", data=data, headers=headers)
        if r.status_code != 201:
            self.stdout.write("There was an error adding this site")
            self.stdout.write(r.text)
            return

        self.stdout.write("This site is now registered at djangoupdater.com")
        self.stdout.write("All went well!")


def hit_updater_view(domain, token):
    base_url = "{site}{url}".format(site=domain, url=reverse("updater_run", kwargs={"token": token}))
    http_url, https_url = "://".join(["http", base_url]), "://".join(["https", base_url])
    if is_reachable_url(https_url + "?health=1"):
        return https_url
    elif is_reachable_url(http_url + "?health=1"):
        return http_url
    return False


def is_reachable_url(url):
    try:
        r = requests.get(url=url, timeout=2.0)
        if r.status_code == 200:
            return True
    except RequestException as e:
        pass
    return False

def get_input(prompt):
    # py2 and py3 compatible input prompt
    if sys.version_info[0] >= 3:
        return input(prompt)
    return raw_input(prompt)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import logging

from django.core.mail.message import EmailMultiAlternatives
from django.template.loader import render_to_string
from django import conf

from .conf import settings

logger = logging.getLogger(__name__)


def send_notification(result):
    """
    Sends a notification.
    :param result: Dictionary containing all updates and security issues
    :return: True if all notifcations have been sent successfully
    """
    # only mails are supported right now. This might change, so we go for the more generic `send_notification`
    # as method name, but use it as a proxy to send_mail
    return send_mail(result, mail_from=conf.settings.SERVER_EMAIL, mail_to=settings.UPDATER_EMAILS)


def send_mail(result, mail_from, mail_to, fail_sitently=False):
    """
    Sends a notification email.
    :param result: Dictionary containing all updates and security issues
    :return: :bool: True if mail has been send successfully
    """

    subject = "Important: Security updates on %s" % result["site"] if result["security"] \
        else "Updates available on %s" % result["site"]
    txt_message = render_to_string("summary.txt", result)
    html_message = render_to_string("summary.html", result)
    mail = EmailMultiAlternatives(
        subject, txt_message, mail_from, mail_to)
    mail.attach_alternative(html_message, 'text/html')
    return mail.send(fail_silently=fail_sitently)


# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import pip
import logging

from django.utils import timezone
from django.contrib.sites.shortcuts import get_current_site

from .conf import settings
from .models import Notification
from .util import send_notification
import requests

logger = logging.getLogger(__name__)


def run_check(token=None):
    """
    Main entrypoint for all update checks. Fetches issues and updates and decides if a notification is sent.
    :param site: (Optional) site token
    :return: True, if a notifaction has been sent. False otherwise
    """
    result = get_updates(token=token)
    notify = False
    if not result["notified"]:
        if result["security"]:
            # if we have a security issue we'll notify no matter what
            notify = True
        elif result["updates"] and settings.UPDATER_NOTIFY_ON_UPDATES:
            # only notify if we a) have updates b) NOTIFY_ON_UPDATES is set c) the last notification is old enough
            delta = timezone.timedelta(days=settings.UPDATER_DAYS_BETWEEN_NOTIFICATION, hours=1)
            if not Notification.objects.filter(created__gte=timezone.now() - delta).exists():
                notify = True

        if notify:
            result["site"] = get_current_site(None)
            send_notification(result)
            Notification.objects.create(security_issue=result["security"] != [])

    return result["notified"] or notify


def get_updates(token=None):
    """
    :return: Dictionary containing all information about all installed packages
    """
    headers = {}
    if settings.UPDATER_TOKEN:
        headers["Authorization"] = "Token " + settings.UPDATER_TOKEN

    data = {"packages": get_packages(), "token": token}
    r = requests.post(settings.UPDATER_BASE_URL + "/api/v1/packages/", json=data, headers=headers)
    return r.json()


def get_packages():
    """
    :return: A dictionary containing installed packages
    """
    return dict((i.key, i.version) for i in pip.get_installed_distributions())


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .views import run_view
from django.conf.urls import url

urlpatterns = [
    url(r'run/(?P<token>[0-9a-f-]{36})/$', run_view, name="updater_run"),
]


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('updater', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('security_issue', models.BooleanField(default=False)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
from django.db import models
from uuid import uuid4
from django.utils import timezone


class Token(models.Model):

    token = models.CharField(max_length=36, default=uuid4)


class Notification(models.Model):

    created = models.DateTimeField(default=timezone.now)
    security_issue = models.BooleanField(default=False)